==> superstatic-6.0.3.tgz: package/lib/middleware/redirects.js <==
/**
 * Copyright 2015 Google Inc. All Rights Reserved.
 * Use of this source code is governed by a BSD-style
 * license that can be found in the LICENSE file or at
 * https://github.com/firebase/superstatic/blob/master/LICENSE
 */
'use strict';

var isUrl = require('is-url');
var _ = require('lodash');

var minimatch = require('minimatch');
var pathToRegexp = require('path-to-regexp');
var slasher = require('glob-slasher');

function formatExternalUrl(u) {
  var cleaned = u
    .replace('/http:/', 'http://')
    .replace('/https:/', 'https://');

  return (isUrl(cleaned)) ? cleaned : u;
}

function addQuery(url, qs) {
  if (url.indexOf('?') >= 0) {
    return url + '&' + qs;
  } else if (qs && qs.length) {
    return url + '?' + qs;
  }
  return url;
}

var Redirect = function(source, destination, type) {
  this.type = type || 301;
  this.source = slasher(source);
  this.destination = destination;

  if (this.destination.match(/(?:^|\/):/)) {
    this.captureKeys = [];
    this.capture = pathToRegexp(this.source, this.captureKeys);
    this.compileDestination = pathToRegexp.compile(this.destination);
  }
};

Redirect.prototype.test = function(url) {
  var qs = '';
  if (url.indexOf('?') >= 0) {
    var parts = url.split('?');
    url = parts[0];
    qs = parts[1];
  }

  var match;
  if (this.capture) {
    match = this.capture.exec(url);
  }
  if (match) {
    var params = {};
    for (var i = 0; i < this.captureKeys.length; i++) {
      var m = match[i + 1];
      if (m && m.indexOf('/') >= 0) {
        m = m.split('/');
      }

      params[this.captureKeys[i].name] = m;
    }

    try {
      var dest = this.compileDestination(params);
      return {
        type: this.type,
        destination: addQuery(dest, qs)
      };
    } catch (e) {
      return undefined;
    }
  } else if (minimatch(url, this.source)) {
    return {
      type: this.type,
      destination: addQuery(this.destination, qs)
    };
  }
  return undefined;
};

module.exports = function() {
  return function(req, res, next) {
    var config = _.get(req, 'superstatic.redirects');
    if (!config) {
      return next();
    }

    var redirects = [];
    if (_.isArray(config)) {
      config.forEach(function(redir) {
        redirects.push(new Redirect(redir.source, redir.destination, redir.type));
      });
    } else {
      throw new Error('redirects provided in an unrecognized format');
    }

    var matcher = function(url) {
      for (var i = 0; i < redirects.length; i++) {
        var result = redirects[i].test(url);
        if (result) { return result; }
      }
      return undefined;
    };

    var match = matcher(req.url);

    if (!match) {
      return next();
    }

    // Remove leading slash of a url
    var redirectUrl = formatExternalUrl(match.destination);

    return res.superstatic.handle({
      redirect: redirectUrl,
      status: match.type
    });
  };
};

==> superstatic-7.0.1.tgz: package/lib/middleware/redirects.js <==
/**
 * Copyright 2015 Google Inc. All Rights Reserved.
 * Use of this source code is governed by a BSD-style
 * license that can be found in the LICENSE file or at
 * https://github.com/firebase/superstatic/blob/master/LICENSE
 */
'use strict';

var isUrl = require('is-url');
var _ = require('lodash');

var patterns = require('../utils/patterns');
var pathToRegexp = require('path-to-regexp');
var slasher = require('glob-slasher');

function formatExternalUrl(u) {
  var cleaned = u
    .replace('/http:/', 'http://')
    .replace('/https:/', 'https://');

  return (isUrl(cleaned)) ? cleaned : u;
}

function addQuery(url, qs) {
  if (url.indexOf('?') >= 0) {
    return url + '&' + qs;
  } else if (qs && qs.length) {
    return url + '?' + qs;
  }
  return url;
}

var Redirect = function(glob, regex, destination, type) {
  this.type = type || 301;
  this.glob = slasher(glob);
  this.regex = regex;
  this.destination = destination;

  if (this.destination.match(/(?:^|\/):/)) {
    this.captureKeys = [];
    if (this.glob) {
      this.engine = 'glob';
      this.capture = pathToRegexp(this.glob, this.captureKeys);
    }
    if (this.regex) {
      this.engine = 'pattern';
      this.capture = patterns.createRaw(this.regex);
    }
    this.compileDestination = pathToRegexp.compile(this.destination);
  }
};

Redirect.prototype.test = function(url) {
  var qs = '';
  if (url.indexOf('?') >= 0) {
    var parts = url.split('?');
    url = parts[0];
    qs = parts[1];
  }

  var match;
  if (this.capture) {
    match = this.capture.exec(url);
  }
  if (match) {
    var params = {};
    if (this.engine === 'glob') {
      for (var i = 0; i < this.captureKeys.length; i++) {
        var m = match[i + 1];
        if (m && m.indexOf('/') >= 0) {
          m = m.split('/');
        }

        params[this.captureKeys[i].name] = m;
      }
    } else {
      for (var j = 0; j < match.length; j++) {
        params[j.toString()] = match[j];
      }
      if (match.groups) {
        params = Object.assign(params, match.groups);
      }
    }

    try {
      var dest = decodeURIComponent(this.compileDestination(params));
      return {
        type: this.type,
        destination: encodeURI(addQuery(dest, qs))
      };
    } catch (e) {
      return undefined;
    }
  } else if (patterns.configMatcher(url, {'glob': this.glob, 'regex': this.regex})) {
    return {
      type: this.type,
      destination: encodeURI(addQuery(this.destination, qs))
    };
  }
  return undefined;
};

module.exports = function() {
  return function(req, res, next) {
    var config = _.get(req, 'superstatic.redirects');
    if (!config) {
      return next();
    }

    var redirects = [];
    if (_.isArray(config)) {
      config.forEach(function(redir) {
        var glob = redir.glob || redir.source;
        redirects.push(new Redirect(glob, redir.regex, redir.destination, redir.type));
      });
    } else {
      throw new Error('redirects provided in an unrecognized format');
    }

    var matcher = function(url) {
      for (var i = 0; i < redirects.length; i++) {
        var result = redirects[i].test(url);
        if (result) { return result; }
      }
      return undefined;
    };

    var match = matcher(req.url);

    if (!match) {
      return next();
    }

    // Remove leading slash of a url
    var redirectUrl = formatExternalUrl(match.destination);

    return res.superstatic.handle({
      redirect: redirectUrl,
      status: match.type
    });
  };
};
